@@ -36,6 +36,22 @@ class Job(JobBase):
         return retval
 
 
+def tag(
+    client: SDKAPI, workspace_slug: str, job_slug: str, tags: List[str]
+) -> List[str]:
+    """Add tags to a job."""
+    response = client.execute(
+        op=_tag_request,
+        workspace_slug=workspace_slug,
+        job_slug=job_slug,
+        tags=tags,
+    )
+    tags = response.get("jobAddTags").get("tags")
+
+    jobs = get(client, job_slug)
+    return jobs[0]
+
+
 _get_jobs = Operation(
     query="""
      query sdk_get_jobs(
@@ -90,6 +106,7 @@ _get_jobs = Operation(
                                     label
                                     fileName
                                     url
+                                    metaSizeBytes
                                     dateCreated
                                     dateUpdated
                                 }
@@ -110,9 +127,17 @@ _get_jobs = Operation(
                                 id
                                 slug
                                 fileName
+                                metaSizeBytes
                                 url
                             }
                         }
+                        dateCreated
+                        dateUpdated
+                        tags {
+                            tag {
+                                tag
+                            }
+                        }
                     }
                 }
             }
@@ -183,9 +208,42 @@ def get(
         edges = jobs_as_dict.get("edges") if jobs_as_dict else None
 
         if edges and len(edges) > 0:
-            new_jobs = list(map(lambda x: Job.from_dict(x.get("node")), edges))
+            new_jobs = []
+            for edge in edges:
+                job_data = edge.get("node", {})
+                # Unpack tags
+                job_tags = job_data.get("tags", [])
+                job_data["tags"] = [
+                    tag.get("tag", {}).get("tag") for tag in job_tags if tag.get("tag")
+                ]
+                new_jobs.append(Job.from_dict(job_data))
             jobs.extend(new_jobs)
 
         cursor = workspace.get("jobs").get("pageInfo").get("endCursor")
         hasNextPage = workspace.get("jobs").get("pageInfo").get("hasNextPage")
     return jobs
+
+
+_tag_request = Operation(
+    query="""
+        mutation jobAddTags(
+            $workspace_slug: String!,
+            $job_slug: String!,
+            $tags: [String!]!,
+            ){
+            jobAddTags(
+                input: {
+                    workspaceSlug: $workspace_slug,
+                    jobSlug: $job_slug,
+                    tags: $tags
+                }
+            ) {
+                tags {
+                    tag {
+                        tag
+                    }
+                }
+            }
+        }
+    """
+)
