@@ -1,27 +1,15 @@
 """client.py."""
 
-import base64
 import importlib.metadata
 import os
-from functools import singledispatchmethod, wraps
-from typing import Any, Callable, Dict, List, Optional, Union
+from enum import Enum
+from functools import singledispatchmethod
+from typing import Any, Dict, List, Optional, Union
 
-import dill
-from strangeworks_core.platform.gql import API, APIInfo
 from strangeworks_core.types.resource import Resource
 
 from strangeworks.core.client import auth, file, jobs, resource, workspace
 from strangeworks.core.client.backends import Backend, get_backend, get_backends
-from strangeworks.core.client.experiments import (
-    Experiment,
-    ExperimentConfiguration,
-    ExperimentInput,
-    Trial,
-    TrialSubmission,
-    get_experiment,
-)
-from strangeworks.core.client.experiments import run as experiment_run
-from strangeworks.core.client.experiments import upload_exp_file
 from strangeworks.core.client.file import File
 from strangeworks.core.client.jobs import Job
 from strangeworks.core.client.rest_client import StrangeworksRestClient
@@ -34,6 +22,13 @@ from strangeworks.platform.gql import SDKAPI
 __version__ = importlib.metadata.version("strangeworks")
 
 
+class TagOperator(Enum):
+    """Logical operators for tags."""
+
+    AND = "AND"
+    OR = "OR"
+
+
 class SWClient:
     """Strangeworks client object."""
 
@@ -173,7 +168,8 @@ class SWClient:
         product_slugs: Optional[List[str]] = None,
         statuses: Optional[List[str]] = None,
         tags: Optional[List[str]] = None,
-    ) -> Job:
+        tag_operator: Optional[TagOperator] = None,
+    ) -> Optional[List[Job]]:
         """Retrieve list of jobs associated with the current workspace account.
 
         Parameters
@@ -189,19 +185,60 @@ class SWClient:
         statuses: Optional[List[str]]
             List of job statuses. Only obs whose statuses match will be returned.
             Defaults to None.
+        tags: Optional[List[str]]
+            List of tags to filter the jobs by. Defaults to None.
+        tag_operator: Optional[TagOperator]
+            The logical operator to use for the tags. Can be either "AND" or "OR".
+            Defaults to None, treating the tags list as an OR operation if
+            multiple tags are provided.
 
         Return
         -------
         : Optional[List[Job]]
             List of jobs or None if there are no jobs that match selection criteria.
         """
-        return jobs.get(
-            client=self.get_sdk_api(),
-            job_slug=slug,
-            product_slugs=product_slugs or [],
-            resource_slugs=resource_slugs or [],
-            statuses=statuses or [],
-            tags=tags or [],
+        tag_operator = TagOperator(tag_operator) if tag_operator else TagOperator.OR
+        if tag_operator == TagOperator.OR:
+            return jobs.get(
+                client=self.get_sdk_api(),
+                job_slug=slug,
+                product_slugs=product_slugs or [],
+                resource_slugs=resource_slugs or [],
+                statuses=statuses or [],
+                tags=tags or [],
+            )
+        elif tag_operator == TagOperator.AND:
+            job_list = [self.jobs(tags=t) for t in tags]
+            slug_list = [[j.slug for j in jobset] for jobset in job_list]
+
+            commonalities = set(slug_list[0])
+            for ii in range(1, len(slug_list)):
+                commonalities &= set(slug_list[ii])
+
+            all_jobs_flat = [job for sublist in job_list for job in sublist]
+            unique_jobs = {job.slug: job for job in all_jobs_flat}.values()
+
+            return [job for job in unique_jobs if job.slug in commonalities]
+        else:
+            raise ValueError("tag_operator must be either 'AND' or 'OR'")
+
+    def add_tags(self, job_slug: str, tags: List[str]) -> List[str]:
+        """Add tags to a job.
+
+        Parameters
+        ----------
+        job_slug: str
+            The slug of the job.
+        tags: List[str]
+            The tags to add to the job.
+
+        Returns
+        -------
+        List[str]
+            The tags linked to the job.
+        """
+        return jobs.tag(
+            self.get_sdk_api(), self.cfg.get_active_profile(), job_slug, tags
         )
 
     def workspace_info(self) -> Workspace:
@@ -296,7 +333,7 @@ class SWClient:
 
         Replaces the deprecated BackendsService.
         """
-        return get_backends(
+        backends: List[Backend] = get_backends(
             client=self.get_sdk_api(),
             product_slugs=product_slugs,
             backend_type_slugs=backend_type_slugs,
@@ -304,6 +341,9 @@ class SWClient:
             backend_tags=backend_tags,
         )
 
+        backends = sorted(backends, key=lambda backend: backend.name)
+        return backends
+
     def get_backend(self, backend_slug: str) -> Backend:
         """Return a single backend by the slug.
 
@@ -327,6 +367,10 @@ class SWClient:
             raise StrangeworksError(f"unable to open {file_path}: {str(e)}")
         else:
             with fd:
+                if self.rest_client is None:
+                    raise StrangeworksError(
+                        "REST client is not initialized. Ensure you have authenticated with the correct API Key.",  # noqa
+                    )
                 self.rest_client.put(signedUrl, data=fd)
         return f
 
@@ -479,7 +523,8 @@ class SWClient:
         candidates = [res for res in resources if res.product.slug == _product_slug]
         if len(candidates) > 1:
             resources_list = [
-                f"  resource_slug: {r.slug}, product_slug: {r.product.slug}"
+                f"  strangeworks.set_resource_for_product("
+                f"resource_slug='{r.slug}', product_slug='{r.product.slug}')"
                 for r in candidates
             ]
 
@@ -488,9 +533,7 @@ class SWClient:
             raise StrangeworksError(
                 message=f"More than one matching resource found for "
                 f"{product_slug}. Please select one:\n"
-                f"{resources_string}\n Then call "
-                "strangeworks.set_resource_for_product"
-                "(resource_slug, product_slug)"
+                f"{resources_string}\n"
             )
         if len(candidates) == 0:
             raise StrangeworksError(
@@ -524,9 +567,10 @@ class SWClient:
         # Check child job for error messages
         if sw_job.child_jobs:
             for child_job in sw_job.child_jobs:
-                for f in child_job.files:
-                    if "error" in f.file_name:
-                        child_files.append(self.rest_client.get(url=f.url))
+                if child_job.files is not None:
+                    for f in child_job.files:
+                        if "error" in f.file_name:
+                            child_files.append(self.rest_client.get(url=f.url))
 
         # For svc-from-callable jobs, error message is in results file
         for f in sw_job.files:
@@ -539,378 +583,11 @@ class SWClient:
 
         if sw_job.child_jobs:
             for child_job in sw_job.child_jobs:
-                for f in child_job.files:
-                    if "result" in f.file_name:
-                        result = self.rest_client.get(url=f.url)
-                        if any("error" in r for r in result.keys()):
-                            child_files.append(result)
+                if child_job.files is not None:
+                    for f in child_job.files:
+                        if "result" in f.file_name:
+                            result = self.rest_client.get(url=f.url)
+                            if any("error" in r for r in result.keys()):
+                                child_files.append(result)
 
         return {"parent_job": parent_files, "child_jobs": child_files}
-
-    def run(
-        self,
-        func: Callable[..., Any],
-        input: Union[ExperimentInput, list[ExperimentInput]],
-        cfg: ExperimentConfiguration = ExperimentConfiguration(),
-        **kwargs,
-    ) -> dict[str, TrialSubmission]:
-        """
-        Run an experiment.
-
-        Parameters
-        ----------
-        func: Callable[..., Any]
-            The function to run.
-        input: Union[ExperimentInput, list[ExperimentInput]]
-            The input to the function.
-        cfg: ExperimentConfiguration
-            The configuration for the experiment.
-
-        Returns
-        -------
-        dict[str, TrialSubmission]
-            The results of the experiment.
-        """
-        a = API(self._url, APIInfo.SDK, api_key=self._key)
-        return experiment_run(a, func, input, cfg, **kwargs)
-
-    def experiment(self, cfg: ExperimentConfiguration = ExperimentConfiguration()):
-        """Generate Callable for running an experiment.
-
-        Parameters
-        ----------
-        cfg: ExperimentConfiguration
-            The configuration for the experiment.
-
-        Returns
-        -------
-        Callable[..., Any]
-            The decorated function.
-
-        Examples
-        --------
-        >>> @sw.experiment()
-        ... def my_experiment():
-        ...     return 42
-        ... my_experiment()
-
-        """
-
-        def decorator(func):
-            @wraps(func)
-            def wrapper(*fargs, **fkwargs):
-                experiment_name = (
-                    cfg.experiment_name if cfg.experiment_name else func.__name__
-                )
-                trial_name = cfg.trial_name if cfg.trial_name else func.__name__
-                return self.run(
-                    func,
-                    ExperimentInput(experiment_name, trial_name, fargs, fkwargs),
-                    cfg,
-                )
-
-            wrapper.sw_exp_og_func = func
-            return wrapper
-
-        return decorator
-
-    def experiment_download(
-        self,
-        experiment_name: str,
-        trials: Union[int, list[str]],
-        files: Union[str, list[str]],
-        jobs: Union[bool, list[str]],
-    ) -> Experiment:
-        """Download an Experiment.
-
-        This gives you the flexiblity to download any number of trials.
-        Particular files or jobs can be downloaded for each trial.
-
-        Parameters
-        ----------
-        experiment_name: str
-            The name of the experiment.
-        trials: Union[int, list[str]]
-            The trials to download.
-            If an int is provided, then the last n trials will be downloaded.
-            If a list of str is provided, then the named trials will be downloaded.
-            The list of str are expected to be trial_name values.
-        files: Union[str, list[str]]
-            The files to download.
-            The str or list of str are expected to be file_name values.
-            If a str is provided, then the file will be downloaded.
-            If a list of str is provided, then the files will be downloaded.
-        jobs: Union[bool, list[str]]
-            The jobs to download.
-            If a bool is provided,
-                then the all jobs related with the trial will be downloaded.
-            If a list of str is provided, then the jobs will be downloaded.
-
-        Returns
-        -------
-        Experiment
-            The experiment.
-
-        Examples
-        --------
-        >>> # Download the last 3 trials of the 'heavy_computation' experiment.
-        >>> # Download the 'user_program_stdout.txt' file for each trial.
-        >>> # Do not include any related jobs.
-        >>> import strangeworks as sw
-        >>> sw.authenticate(...) # update with your api key.
-        >>> sw.experiment_download(
-        ...     "heavy_computation", 3, "user_program_stdout.txt", False
-        ... )
-        Experiment(
-            name='heavy_computation',
-            trials=[
-                Trial(
-                    name='siete_ocho_207',
-                    files=[
-                        File(
-                            id=None,
-                            slug='evtstr5wp',
-                            label=None,
-                            file_name='user_program_stdout.txt',
-                            url='https://api.strangeworks.com/files/experiments/lucky-badger-8981-krjdisf9d/trial/snarling-roe-0728-t0k8btudx/evtstr5wp',
-                            content='The answer is 42.',
-                        )
-                    ],
-                    jobs=None,
-                ),
-                Trial(
-                    name='siete_ocho_206',
-                    files=[
-                        File(
-                            id=None,
-                            slug='9kemsahmh',
-                            label=None,
-                            file_name='user_program_stdout.txt',
-                            url='https://api.strangeworks.com/files/experiments/lucky-badger-8981-krjdisf9d/trial/glistening-river-1629-5pjcb0r9j/9kemsahmh',
-                            content='sent request to strangeworks optimization.',
-                            )
-                    ],
-                    jobs=None,
-                ),
-                Trial(
-                    name='siete_ocho_205',
-                    files=[
-                        File(
-                            id=None,
-                            slug='rbm85i4wy',
-                            label=None,
-                            file_name='user_program_stdout.txt',
-                            url='https://api.strangeworks.com/files/experiments/lucky-badger-8981-krjdisf9d/trial/winding-water-8578-giugxm4gd/rbm85i4wy',
-                            content='',
-                        )
-                    ],
-                    jobs=None,
-                )])
-
-        >>> # Download a list of named trials of the 'heavy_computation' experiment.
-        >>> # Download the 'result.pickle' and 'user_program_stdout.txt'
-        >>> #   files for each trial.
-        >>> # If the trial has the files you requested
-        >>> #   then they will be included in the Trial object.
-        >>> # Include any related jobs.
-        >>> # If no jobs are related to any of the trials,
-        >>> #   then the jobs attribute will be None.
-        >>> import strangeworks as sw
-        >>> sw.authenticate(...) # update with your api key.
-        >>> sw.experiment_download(
-        ...     "heavy_computation",
-        ...     ["siete_ocho", "siete_ocho_9"],
-        ...     ["result.pickle", "user_program_stdout.txt"],
-        ...     True,
-        ... )
-        Experiment(
-            name='heavy_computation',
-            trials=[
-                Trial(
-                    name='siete_ocho_9',
-                    files=[
-                        File(
-                            id=None,
-                            slug='hseuqyjd7',
-                            label=None,
-                            file_name='result.pickle',
-                            url='https://api.strangeworks.com/files/experiments/lucky-badger-8981-krjdisf9d/trial/bold-whale-5839-5yhcpjuhz/hseuqyjd7',
-                            content='gASVBAAAAAAAAABNagQu'
-                        ),
-                        File(
-                            id=None,
-                            slug='ey5szvabe',
-                            label=None,
-                            file_name='user_program_stdout.txt',
-                            url='https://api.strangeworks.com/files/experiments/lucky-badger-8981-krjdisf9d/trial/bold-whale-5839-5yhcpjuhz/ey5szvabe',
-                            content=''),
-                    ],
-                    jobs=[
-                        Job(
-                            slug='straight-mountain-3370',
-                            child_jobs=[],
-                            external_identifier='',
-                            resource=None,
-                            status='COMPLETED',
-                            is_terminal_state=None,
-                            remote_status=None,
-                            job_data_schema=None,
-                            job_data=None,
-                            files=[
-                                File(
-                                    id=None,
-                                    slug='7ab5s545s',
-                                    label=None,
-                                    file_name='result.json',
-                                    url='https://api.strangeworks.com/files/jobs/straight-mountain-3370/7ab5s545s/result.json',
-                                    content=None,
-                                )
-                            ],
-                        )
-                    ]),
-                Trial(name='siete_ocho', files=[], jobs=None)
-            ],
-        )
-
-
-
-        """
-        a = API(self._url, APIInfo.SDK, api_key=self._key)
-        exp = get_experiment(a, experiment_name, trials, files, jobs)
-
-        ts = []
-        for t in exp.trials:
-            fs = []
-            for f in t.files:
-                try:
-                    content = self.rest_client.get(url=f.url)
-                    if (
-                        f.file_name == "result.pickle"
-                        and f.data_schema_slug == "sw-result-pickle"
-                    ):
-                        content = dill.loads(base64.b64decode(content))
-                except Exception as e:
-                    content = e
-                fs.append(
-                    File(
-                        id=f.id,
-                        slug=f.slug,
-                        label=f.label,
-                        file_name=f.file_name,
-                        url=f.url,
-                        content=content,
-                    )
-                )
-            ts.append(Trial(name=t.name, status=t.status, files=fs, jobs=t.jobs))
-
-        return Experiment(
-            exp.name,
-            ts,
-        )
-
-    def experiment_upload(
-        self,
-        experiment_name: str,
-        trial_name: str,
-        file_path: str,
-    ) -> File:
-        """
-        experiment_upload allows you to upload a file to a trial.
-
-        Parameters
-        ----------
-        experiment_name: str
-            The name of the experiment.
-        trial_name: str
-            The name of the trial.
-        file_path: str
-            The path to the file to upload.
-
-        Returns
-        -------
-        File
-            The file that was uploaded.
-
-        Examples
-        --------
-        >>> import strangeworks as sw
-        >>> sw.authenticate(...)
-        >>> sw.experiment_upload(
-        ...     "heavy_computation",
-        ...     "siete_ocho_9",
-        ...     "./post_processed_result.json",
-        ... )
-
-        """
-        a = API(self._url, APIInfo.SDK, api_key=self._key)
-        f, url = upload_exp_file(a, experiment_name, trial_name, file_path)
-        try:
-            fd = open(file_path, "rb")
-        except IOError as e:
-            raise StrangeworksError(f"unable to open {file_path}: {str(e)}")
-        else:
-            with fd:
-                self.rest_client.put(url, data=fd)
-        return f
-
-    def experiment_get_result(self, name: str, trial: Optional[str] = None) -> Any:
-        """Get Experiment Result.
-
-        Allows you to download the result of an experiment.
-
-        Assumes that your current environment has the same modules loaded / installed
-        as the environment which you submitted the experiment.
-
-        Parameters
-        ----------
-        name: str
-            The name of the experiment.
-        trial: Optional[str]
-            The name of the trial. Defaults to the last trial in the experiment.
-
-        Returns
-        -------
-        Any
-            The result of the experiment.
-            This is the value that you returned from your experiment function.
-
-        Examples
-        --------
-        >>> import strangeworks as sw
-        >>> sw.authenticate(...)
-        >>> sw.experiment_get_result("heavy_computation", "siete_ocho_9")
-        42
-        >>> sw.experiment_get_result("heavy_computation")
-        42
-
-        Raises
-        ------
-        StrangeworksError
-            If we are unable to download the result.
-        StrangeworksError
-            If we are unable to get one trial from the experiment.
-        StrangeworksError
-            If the trial is not in a COMPLETED state.
-        StrangeworksError
-            If we are unable to get the one result file from the trial.
-        """
-        t = [trial] if trial else 1
-        exp = self.experiment_download(name, t, "result.pickle", False)
-        if len(exp.trials) != 1:
-            raise StrangeworksError(
-                f"we are unable to grab the {t} trial from the experiment"
-            )
-
-        exp_trial = exp.trials[0]
-        if not exp_trial.status or (exp_trial.status != "COMPLETED"):
-            raise StrangeworksError(
-                f"Trial {exp_trial.name} is not in a COMPLETED state. "
-                f"Current state is {exp_trial.status}."
-            )
-        if len(exp_trial.files) != 1:
-            raise StrangeworksError(
-                f"Trial {exp_trial.name} does not have exactly one file. "
-                f"Current number of files is {len(exp_trial.files)}."
-            )
-
-        return exp_trial.files[0].content
